package org.nocrala.tools.texttablefmt;

import org.nocrala.tools.utils.Filler;

/**
 * <p> Border style define the actual character(s) that will be rendered as borders,
 * when a border or separator is shown.</p>
 * 
 * <p>You can use predefined styles or custom styles.</p>
 * 
 * <h3>Predefined Styles</h3>
 * 
 * <p>The following border styles are already predefined and can be used out of
 * the box:</p>
 * 
 * <p><b>Note:</b> Styles starting with <b>UNICODE_</b> use characters (codepoints)
 * outside the ASCII and ISO-8859-1 (Latin-1) sets. Therefore, some Operating Systems or
 * tools might not be able to display them correctly. Avoid these predefined
 * tiles if you need maximum compatibility.</p>
 * 
 *  <p><b>BorderStyle.CLASSIC</b></p>
 *  <pre class='example'>
 *
 *  +---------+-----------+----------+--------+
 *  |Country  | Population|Area (km2)| Density|
 *  +---------+----------------------+--------+
 *  |Special 1|         1.350.000.000|   16.67|
 *  +---------+----------------------+--------+
 *  |Total    |147 000 000| 9 250 000|   15.89|
 *  +---------+-----------+----------+--------+
 *  </pre>
 *
 *  <p><b>BorderStyle.BLANKS</b></p>
 *  <pre class='example'>
 *
 *                                             
 *   Country    Population Area (km2)  Density 
 *                                             
 *   Special 1          1.350.000.000    16.67 
 *                                             
 *   Total     147 000 000  9 250 000    15.89 
 *                                             
 *  </pre>
 *
 *  <p><b>BorderStyle.DOTS</b></p>
 *  <pre class='example'>
 *
 *  ...........................................
 *  .Country  . Population.Area (km2). Density.
 *  ...........................................
 *  .Special 1.         1.350.000.000.   16.67.
 *  ...........................................
 *  .Total    .147 000 000. 9 250 000.   15.89.
 *  ...........................................
 *  </pre>
 *
 *  <p><b>BorderStyle.ASTERISKS</b></p>
 *  <pre class='example'>
 *
 *  *******************************************
 *  *Country  * Population*Area (km2)* Density*
 *  *******************************************
 *  *Special 1*         1.350.000.000*   16.67*
 *  *******************************************
 *  *Total    *147 000 000* 9 250 000*   15.89*
 *  *******************************************
 *  </pre>
 *
 *  <p><b>BorderStyle.CLASSIC_WIDE</b></p>
 *  <pre class='example'>
 *
 *  +-----------+-------------+------------+----------+
 *  | Country   |  Population | Area (km2) |  Density |
 *  +-----------+--------------------------+----------+
 *  | Special 1 |            1.350.000.000 |    16.67 |
 *  +-----------+--------------------------+----------+
 *  | Total     | 147 000 000 |  9 250 000 |    15.89 |
 *  +-----------+-------------+------------+----------+
 *  </pre>
 *
 *  <p><b>BorderStyle.CLASSIC_LIGHT</b></p>
 *  <pre class='example'>
 *
 *  +-----------------------------------------+
 *  |Country    Population Area (km2)  Density|
 *  +-----------------------------------------+
 *  |Special 1          1.350.000.000    16.67|
 *  +-----------------------------------------+
 *  |Total     147 000 000  9 250 000    15.89|
 *  +-----------------------------------------+
 *  </pre>
 *
 *  <p><b>BorderStyle.CLASSIC_LIGHT_WIDE</b></p>
 *  <pre class='example'>
 *
 *  +----------------------------------------------+
 *  | Country     Population  Area (km2)   Density |
 *  +----------------------------------------------+
 *  | Special 1            1.350.000.000     16.67 |
 *  +----------------------------------------------+
 *  | Total      147 000 000   9 250 000     15.89 |
 *  +----------------------------------------------+
 *  </pre>
 *
 *  <p><b>BorderStyle.HORIZONTAL_ONLY</b></p>
 *  <pre class='example'>
 *
 *  --------- ----------- ---------- --------
 *  Country    Population Area (km2)  Density
 *  --------- ----------- ---------- --------
 *  Special 1          1.350.000.000    16.67
 *  --------- ----------- ---------- --------
 *  Total     147 000 000  9 250 000    15.89
 *  --------- ----------- ---------- --------
 *  </pre>
 *
 *  <p><b>BorderStyle.CLASSIC_COMPATIBLE</b></p>
 *  <pre class='example'>
 *
 *  +---------+-----------+----------+--------+
 *  !Country  ! Population!Area (km2)! Density!
 *  +---------+----------------------+--------+
 *  !Special 1!         1.350.000.000!   16.67!
 *  +---------+----------------------+--------+
 *  !Total    !147 000 000! 9 250 000!   15.89!
 *  +---------+-----------+----------+--------+
 *  </pre>
 *
 *  <p><b>BorderStyle.CLASSIC_COMPATIBLE_WIDE</b></p>
 *  <pre class='example'>
 *
 *  +-----------+-------------+------------+----------+
 *  ! Country   !  Population ! Area (km2) !  Density !
 *  +-----------+--------------------------+----------+
 *  ! Special 1 !            1.350.000.000 !    16.67 !
 *  +-----------+--------------------------+----------+
 *  ! Total     ! 147 000 000 !  9 250 000 !    15.89 !
 *  +-----------+-------------+------------+----------+
 *  </pre>
 *
 *  <p><b>BorderStyle.CLASSIC_COMPATIBLE_LIGHT_WIDE</b></p>
 *  <pre class='example'>
 *
 *  +----------------------------------------------+
 *  ! Country     Population  Area (km2)   Density !
 *  +----------------------------------------------+
 *  ! Special 1            1.350.000.000     16.67 !
 *  +----------------------------------------------+
 *  ! Total      147 000 000   9 250 000     15.89 !
 *  +----------------------------------------------+
 *  </pre>
 *
 *  <p><b>BorderStyle.HEAVY</b></p>
 *  <pre class='example'>
 *
 *  +=========================================+
 *  |Country  | Population|Area (km2)| Density|
 *  +=========================================+
 *  |Special 1|         1.350.000.000|   16.67|
 *  +=========================================+
 *  |Total    |147 000 000| 9 250 000|   15.89|
 *  +=========================================+
 *  </pre>
 *
 *  <p><b>BorderStyle.HEAVY_TOP_AND_BOTTOM</b></p>
 *  <pre class='example'>
 *
 *  +=========================================+
 *  |Country  | Population|Area (km2)| Density|
 *  +-----------------------------------------+
 *  |Special 1|         1.350.000.000|   16.67|
 *  +-----------------------------------------+
 *  |Total    |147 000 000| 9 250 000|   15.89|
 *  +=========================================+
 *  </pre>
 *
 *  <p><b>BorderStyle.DESIGN_CAFE</b></p>
 *  <pre class='example'>
 *
 *  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  Country    Population Area (km2)  Density
 *  ~~~~~~~~~ ~~~~~~~~~~~ ~~~~~~~~~~ ~~~~~~~~
 *  Special 1          1.350.000.000    16.67
 *  ~~~~~~~~~ ~~~~~~~~~~~ ~~~~~~~~~~ ~~~~~~~~
 *  Total     147 000 000  9 250 000    15.89
 *  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  </pre>
 *
 *  <p><b>BorderStyle.DESIGN_CASUAL</b></p>
 *  <pre class='example'>
 *
 *  =========================================
 *  Country    Population Area (km2)  Density
 *  ~~~~~~~~~ ~~~~~~~~~~~ ~~~~~~~~~~ ~~~~~~~~
 *  Special 1          1.350.000.000    16.67
 *  ~~~~~~~~~ ~~~~~~~~~~~ ~~~~~~~~~~ ~~~~~~~~
 *  Total     147 000 000  9 250 000    15.89
 *  =========================================
 *  </pre>
 *
 *  <p><b>BorderStyle.DESIGN_FORMAL</b></p>
 *  <pre class='example'>
 *
 *  =========================================
 *  Country    Population Area (km2)  Density
 *  --------- ----------- ---------- --------
 *  Special 1          1.350.000.000    16.67
 *  --------- ----------- ---------- --------
 *  Total     147 000 000  9 250 000    15.89
 *  =========================================
 *  </pre>
 *
 *  <p><b>BorderStyle.DESIGN_FORMAL_INVERSE</b></p>
 *  <pre class='example'>
 *
 *  -----------------------------------------
 *  Country    Population Area (km2)  Density
 *  ========= =========== ========== ========
 *  Special 1          1.350.000.000    16.67
 *  ========= =========== ========== ========
 *  Total     147 000 000  9 250 000    15.89
 *  -----------------------------------------
 *  </pre>
 *
 *  <p><b>BorderStyle.DESIGN_SLASH</b></p>
 *  <pre class='example'>
 *
 *  /////////////////////////////////////////
 *  Country    Population Area (km2)  Density
 *  --------- ----------- ---------- --------
 *  Special 1          1.350.000.000    16.67
 *  --------- ----------- ---------- --------
 *  Total     147 000 000  9 250 000    15.89
 *  /////////////////////////////////////////
 *  </pre>
 *
 *  <p><b>BorderStyle.DESIGN_TUBES</b></p>
 *  <pre class='example'>
 *
 *   _________________________________________ 
 *  |Country  | Population|Area (km2)| Density|
 *  |_________|______________________|________|
 *  |Special 1|         1.350.000.000|   16.67|
 *  |_________|______________________|________|
 *  |Total    |147 000 000| 9 250 000|   15.89|
 *  |_________|___________|__________|________|
 *  </pre>
 *
 *  <p><b>BorderStyle.DESIGN_DIM</b></p>
 *  <pre class='example'>
 *
 *  &#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;
 *  Country    Population Area (km2)  Density
 *  &#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168; &#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168; &#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;
 *  Special 1          1.350.000.000    16.67
 *  &#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168; &#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168; &#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;
 *  Total     147 000 000  9 250 000    15.89
 *  &#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;
 *  </pre>
 *
 *  <p><b>BorderStyle.DESIGN_DOTS</b></p>
 *  <pre class='example'>
 *
 *  &#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;
 *  &#183;Country  &#183; Population&#183;Area (km2)&#183; Density&#183;
 *  &#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;
 *  &#183;Special 1&#183;         1.350.000.000&#183;   16.67&#183;
 *  &#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;
 *  &#183;Total    &#183;147 000 000&#183; 9 250 000&#183;   15.89&#183;
 *  &#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;
 *  </pre>
 *
 *  <p><b>BorderStyle.DESIGN_CURTAIN</b></p>
 *  <pre class='example'>
 *
 *  o~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~o
 *    Country    Population Area (km2)  Density  
 *    --------- ----------- ---------- --------  
 *    Special 1          1.350.000.000    16.67  
 *    --------- ----------- ---------- --------  
 *    Total     147 000 000  9 250 000    15.89  
 *  o~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~o
 *  </pre>
 *
 *  <p><b>BorderStyle.DESIGN_CURTAIN_HEAVY</b></p>
 *  <pre class='example'>
 *
 *  o===========================================o
 *    Country    Population Area (km2)  Density  
 *    --------- ----------- ---------- --------  
 *    Special 1          1.350.000.000    16.67  
 *    --------- ----------- ---------- --------  
 *    Total     147 000 000  9 250 000    15.89  
 *  o===========================================o
 *  </pre>
 *
 *  <p><b>BorderStyle.DESIGN_PAPYRUS</b></p>
 *  <pre class='example'>
 *
 *  o~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~o
 *   )  Country     Population  Area (km2)   Density  ( 
 *   )  ~~~~~~~~~  ~~~~~~~~~~~  ~~~~~~~~~~  ~~~~~~~~  ( 
 *   )  Special 1            1.350.000.000     16.67  ( 
 *   )  ~~~~~~~~~  ~~~~~~~~~~~  ~~~~~~~~~~  ~~~~~~~~  ( 
 *   )  Total      147 000 000   9 250 000     15.89  ( 
 *  o~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~o
 *  </pre>
 *
 *  <p><b>BorderStyle.DESIGN_CAFE_WIDE</b></p>
 *  <pre class='example'>
 *
 *  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  Country     Population  Area (km2)   Density
 *  ~~~~~~~~~  ~~~~~~~~~~~  ~~~~~~~~~~  ~~~~~~~~
 *  Special 1            1.350.000.000     16.67
 *  ~~~~~~~~~  ~~~~~~~~~~~  ~~~~~~~~~~  ~~~~~~~~
 *  Total      147 000 000   9 250 000     15.89
 *  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  </pre>
 *
 *  <p><b>BorderStyle.DESIGN_CASUAL_WIDE</b></p>
 *  <pre class='example'>
 *
 *  ============================================
 *  Country     Population  Area (km2)   Density
 *  ~~~~~~~~~  ~~~~~~~~~~~  ~~~~~~~~~~  ~~~~~~~~
 *  Special 1            1.350.000.000     16.67
 *  ~~~~~~~~~  ~~~~~~~~~~~  ~~~~~~~~~~  ~~~~~~~~
 *  Total      147 000 000   9 250 000     15.89
 *  ============================================
 *  </pre>
 *
 *  <p><b>BorderStyle.DESIGN_FORMAL_WIDE</b></p>
 *  <pre class='example'>
 *
 *  ============================================
 *  Country     Population  Area (km2)   Density
 *  ---------  -----------  ----------  --------
 *  Special 1            1.350.000.000     16.67
 *  ---------  -----------  ----------  --------
 *  Total      147 000 000   9 250 000     15.89
 *  ============================================
 *  </pre>
 *
 *  <p><b>BorderStyle.DESIGN_FORMAL_INVERSE_WIDE</b></p>
 *  <pre class='example'>
 *
 *  --------------------------------------------
 *  Country     Population  Area (km2)   Density
 *  =========  ===========  ==========  ========
 *  Special 1            1.350.000.000     16.67
 *  =========  ===========  ==========  ========
 *  Total      147 000 000   9 250 000     15.89
 *  --------------------------------------------
 *  </pre>
 *
 *  <p><b>BorderStyle.DESIGN_SLASH_WIDE</b></p>
 *  <pre class='example'>
 *
 *  ////////////////////////////////////////////
 *  Country     Population  Area (km2)   Density
 *  ---------  -----------  ----------  --------
 *  Special 1            1.350.000.000     16.67
 *  ---------  -----------  ----------  --------
 *  Total      147 000 000   9 250 000     15.89
 *  ////////////////////////////////////////////
 *  </pre>
 *
 *  <p><b>BorderStyle.DESIGN_TUBES_WIDE</b></p>
 *  <pre class='example'>
 *
 *   _________________________________________________ 
 *  | Country   |  Population | Area (km2) |  Density |
 *  |___________|__________________________|__________|
 *  | Special 1 |            1.350.000.000 |    16.67 |
 *  |___________|__________________________|__________|
 *  | Total     | 147 000 000 |  9 250 000 |    15.89 |
 *  |___________|_____________|____________|__________|
 *  </pre>
 *
 *  <p><b>BorderStyle.DESIGN_DIM_WIDE</b></p>
 *  <pre class='example'>
 *
 *  &#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;
 *  Country     Population  Area (km2)   Density
 *  &#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;  &#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;  &#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;  &#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;
 *  Special 1            1.350.000.000     16.67
 *  &#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;  &#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;  &#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;  &#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;
 *  Total      147 000 000   9 250 000     15.89
 *  &#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;&#168;
 *  </pre>
 *
 *  <p><b>BorderStyle.DESIGN_DOTS_WIDE</b></p>
 *  <pre class='example'>
 *
 *  &#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;
 *  &#183; Country   &#183;  Population &#183; Area (km2) &#183;  Density &#183;
 *  &#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183; &#183;
 *  &#183; Special 1 &#183;            1.350.000.000 &#183;    16.67 &#183;
 *  &#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183; &#183;
 *  &#183; Total     &#183; 147 000 000 &#183;  9 250 000 &#183;    15.89 &#183;
 *  &#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;&#183;
 *  </pre>
 *
 *  <p><b>BorderStyle.DESIGN_CURTAIN_WIDE</b></p>
 *  <pre class='example'>
 *
 *  o~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~o
 *    Country     Population  Area (km2)   Density  
 *    ---------  -----------  ----------  --------  
 *    Special 1            1.350.000.000     16.67  
 *    ---------  -----------  ----------  --------  
 *    Total      147 000 000   9 250 000     15.89  
 *  o~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~o
 *  </pre>
 *
 *  <p><b>BorderStyle.DESIGN_CURTAIN_HEAVY_WIDE</b></p>
 *  <pre class='example'>
 *
 *  o==============================================o
 *    Country     Population  Area (km2)   Density  
 *    ---------  -----------  ----------  --------  
 *    Special 1            1.350.000.000     16.67  
 *    ---------  -----------  ----------  --------  
 *    Total      147 000 000   9 250 000     15.89  
 *  o==============================================o
 *  </pre>
 *
 *  <p><b>BorderStyle.UNICODE_BOX</b></p>
 *  <pre class='example'>
 *
 *  &#9484;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9516;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9516;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9516;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9488;
 *  &#9474;Country  &#9474; Population&#9474;Area (km2)&#9474; Density&#9474;
 *  &#9500;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9524;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9508;
 *  &#9474;Special 1&#9474;         1.350.000.000&#9474;   16.67&#9474;
 *  &#9500;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9516;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9508;
 *  &#9474;Total    &#9474;147 000 000&#9474; 9 250 000&#9474;   15.89&#9474;
 *  &#9492;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9524;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9524;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9524;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9496;
 *  </pre>
 *
 *  <p><b>BorderStyle.UNICODE_BOX_DOUBLE_BORDER</b></p>
 *  <pre class='example'>
 *
 *  &#9556;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9572;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9572;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9572;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9559;
 *  &#9553;Country  &#9474; Population&#9474;Area (km2)&#9474; Density&#9553;
 *  &#9567;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9524;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9570;
 *  &#9553;Special 1&#9474;         1.350.000.000&#9474;   16.67&#9553;
 *  &#9567;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9516;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9570;
 *  &#9553;Total    &#9474;147 000 000&#9474; 9 250 000&#9474;   15.89&#9553;
 *  &#9562;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9575;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9575;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9575;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9565;
 *  </pre>
 *
 *  <p><b>BorderStyle.UNICODE_BOX_HEAVY_BORDER</b></p>
 *  <pre class='example'>
 *
 *  &#9487;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9519;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9519;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9519;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9491;
 *  &#9475;Country  &#9474; Population&#9474;Area (km2)&#9474; Density&#9475;
 *  &#9504;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9524;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9512;
 *  &#9475;Special 1&#9474;         1.350.000.000&#9474;   16.67&#9475;
 *  &#9504;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9516;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9512;
 *  &#9475;Total    &#9474;147 000 000&#9474; 9 250 000&#9474;   15.89&#9475;
 *  &#9495;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9527;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9527;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9527;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9499;
 *  </pre>
 *
 *  <p><b>BorderStyle.UNICODE_DOUBLE_BOX</b></p>
 *  <pre class='example'>
 *
 *  &#9556;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9574;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9574;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9574;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9559;
 *  &#9553;Country  &#9553; Population&#9553;Area (km2)&#9553; Density&#9553;
 *  &#9568;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9580;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9577;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9580;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9571;
 *  &#9553;Special 1&#9553;         1.350.000.000&#9553;   16.67&#9553;
 *  &#9568;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9580;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9574;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9580;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9571;
 *  &#9553;Total    &#9553;147 000 000&#9553; 9 250 000&#9553;   15.89&#9553;
 *  &#9562;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9577;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9577;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9577;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9565;
 *  </pre>
 *
 *  <p><b>BorderStyle.UNICODE_HEAVY_BOX</b></p>
 *  <pre class='example'>
 *
 *  &#9487;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9523;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9523;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9523;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9491;
 *  &#9475;Country  &#9475; Population&#9475;Area (km2)&#9475; Density&#9475;
 *  &#9507;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9547;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9531;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9547;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9515;
 *  &#9475;Special 1&#9475;         1.350.000.000&#9475;   16.67&#9475;
 *  &#9507;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9547;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9523;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9547;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9515;
 *  &#9475;Total    &#9475;147 000 000&#9475; 9 250 000&#9475;   15.89&#9475;
 *  &#9495;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9531;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9531;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9531;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9499;
 *  </pre>
 *
 *  <p><b>BorderStyle.UNICODE_ROUND_BOX</b></p>
 *  <pre class='example'>
 *
 *  &#9581;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9516;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9516;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9516;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9582;
 *  &#9474;Country  &#9474; Population&#9474;Area (km2)&#9474; Density&#9474;
 *  &#9500;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9524;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9508;
 *  &#9474;Special 1&#9474;         1.350.000.000&#9474;   16.67&#9474;
 *  &#9500;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9516;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9508;
 *  &#9474;Total    &#9474;147 000 000&#9474; 9 250 000&#9474;   15.89&#9474;
 *  &#9584;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9524;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9524;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9524;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9583;
 *  </pre>
 *
 *  <p><b>BorderStyle.UNICODE_BOX_WIDE</b></p>
 *  <pre class='example'>
 *
 *  &#9484;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9516;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9516;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9516;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9488;
 *  &#9474; Country   &#9474;  Population &#9474; Area (km2) &#9474;  Density &#9474;
 *  &#9500;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9524;  &#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9508;
 *  &#9474; Special 1 &#9474;            1.350.000.000 &#9474;    16.67 &#9474;
 *  &#9500;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9516;  &#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9508;
 *  &#9474; Total     &#9474; 147 000 000 &#9474;  9 250 000 &#9474;    15.89 &#9474;
 *  &#9492;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9524;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9524;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9524;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9496;
 *  </pre>
 *
 *  <p><b>BorderStyle.UNICODE_ROUND_BOX_WIDE</b></p>
 *  <pre class='example'>
 *
 *  &#9581;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9516;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9516;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9516;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9582;
 *  &#9474; Country   &#9474;  Population &#9474; Area (km2) &#9474;  Density &#9474;
 *  &#9500;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9524;  &#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9508;
 *  &#9474; Special 1 &#9474;            1.350.000.000 &#9474;    16.67 &#9474;
 *  &#9500;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9516;  &#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9508;
 *  &#9474; Total     &#9474; 147 000 000 &#9474;  9 250 000 &#9474;    15.89 &#9474;
 *  &#9584;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9524;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9524;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9524;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9583;
 *  </pre>
 *
 *  <p><b>BorderStyle.UNICODE_HEAVY_BOX_WIDE</b></p>
 *  <pre class='example'>
 *
 *  &#9487;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9523;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9523;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9523;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9491;
 *  &#9475; Country   &#9475;  Population &#9475; Area (km2) &#9475;  Density &#9475;
 *  &#9507;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9547;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9524;  &#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9547;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9515;
 *  &#9475; Special 1 &#9475;            1.350.000.000 &#9475;    16.67 &#9475;
 *  &#9507;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9547;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9516;  &#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9547;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9515;
 *  &#9475; Total     &#9475; 147 000 000 &#9475;  9 250 000 &#9475;    15.89 &#9475;
 *  &#9495;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9531;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9531;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9531;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9499;
 *  </pre>
 *
 *  <p><b>BorderStyle.UNICODE_BOX_HEAVY_BORDER_WIDE</b></p>
 *  <pre class='example'>
 *
 *  &#9487;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9519;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9519;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9519;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9491;
 *  &#9475; Country   &#9474;  Population &#9474; Area (km2) &#9474;  Density &#9475;
 *  &#9504;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9473;&#9524;&#9473;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9512;
 *  &#9475; Special 1 &#9474;            1.350.000.000 &#9474;    16.67 &#9475;
 *  &#9504;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9473;&#9516;&#9473;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9512;
 *  &#9475; Total     &#9474; 147 000 000 &#9474;  9 250 000 &#9474;    15.89 &#9475;
 *  &#9495;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9527;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9527;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9527;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9473;&#9499;
 *  </pre>
 *
 *  <p><b>BorderStyle.UNICODE_DOUBLE_BOX_WIDE</b></p>
 *  <pre class='example'>
 *
 *  &#9556;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9574;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9574;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9574;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9559;
 *  &#9553; Country   &#9553;  Population &#9553; Area (km2) &#9553;  Density &#9553;
 *  &#9568;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9580;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9577;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9580;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9571;
 *  &#9553; Special 1 &#9553;            1.350.000.000 &#9553;    16.67 &#9553;
 *  &#9568;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9580;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9574;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9580;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9571;
 *  &#9553; Total     &#9553; 147 000 000 &#9553;  9 250 000 &#9553;    15.89 &#9553;
 *  &#9562;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9577;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9577;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9577;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9565;
 *  </pre>
 *
 *  <p><b>BorderStyle.UNICODE_BOX_DOUBLE_BORDER_WIDE</b></p>
 *  <pre class='example'>
 *
 *  &#9556;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9572;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9572;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9572;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9559;
 *  &#9553; Country   &#9474;  Population &#9474; Area (km2) &#9474;  Density &#9553;
 *  &#9567;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9473;&#9524;&#9473;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9570;
 *  &#9553; Special 1 &#9474;            1.350.000.000 &#9474;    16.67 &#9553;
 *  &#9567;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9473;&#9516;&#9473;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9532;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9472;&#9570;
 *  &#9553; Total     &#9474; 147 000 000 &#9474;  9 250 000 &#9474;    15.89 &#9553;
 *  &#9562;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9575;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9575;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9575;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9552;&#9565;
 *  </pre>
 * 
 * <p>If no predefined style fits your needs, you can use a...</p>
 * 
 * <h3>Custom Styles</h3>
 * 
 * <p>To define a custom border style you need to specify all 15 character tiles used for the table 
 * rendering using one of the BorderStyle constructors.</p>
 * 
 * <p>For single-character tiles use the constructor:</p>
 * 
 * <pre>
 *   new BorderStyle("1a234b567c89xyz")</pre>
 * 
 * <p>This border style will generate a table like...</p>
 * 
 * <pre class='example'>
 * 
 *  1aaaaaaaaa2aaaaaaaaaaa2aaaaaaaaaa2aaaaaaa3
 *  xCountry  y PopulationyArea (km2)yDensityz
 *  4bbbbbbbbb5bbbbbbbbbbb5bbbbbbbbbb5bbbbbbb6
 *  xChile    y 17 000 000y 1 250 000y  13.60z
 *  xArgentinay 50 000 000y 3 000 000y  16.67z
 *  xBrasil   y 80 000 000y 5 000 000y  16.00z
 *  4bbbbbbbbb5bbbbbbbbbbb5bbbbbbbbbb5bbbbbbb6
 *  xTotal    y147 000 000y 9 250 000y  15.89z
 *  7ccccccccc8ccccccccccc8cccccccccc8ccccccc9
 *  </pre>
 * 
 * <p>For multi-character tiles use the constructor:</p>
 * 
 * <pre>
 *   new BorderStyle("+=", "=", "==", "=+", "+=", "=", 
 *     "==", "=+", "+=", "=", "==", "=+", "| ", "  ", " |");</pre>
 * 
 * <p>This border style will generate a table like...</p>
 * 
 * <pre class='example'>
 * 
 *  +=============================================+
 *  | Country     Population  Area (km2)  Density |
 *  +=============================================+
 *  | Chile       17 000 000   1 250 000    13.60 |
 *  | Argentina   50 000 000   3 000 000    16.67 |
 *  | Brasil      80 000 000   5 000 000    16.00 |
 *  +=============================================+
 *  | Total      147 000 000   9 250 000    15.89 |
 *  +=============================================+
 *  </pre>
 * 
 * <p>Use any tiles that fits your needs.</p>
 * 
 */
public class BorderStyle {

  // Demo tile

  public static final BorderStyle DEMO = new BorderStyle("1a234b567c89xyztu");

  // Basic tiles

  public static final BorderStyle BLANKS = new BorderStyle("    " + "    "
      + "    " + "   " + "  ");

  public static final BorderStyle DOTS = new BorderStyle("...." + "...."
      + "...." + "..." + "..");

  public static final BorderStyle ASTERISKS = new BorderStyle("****" + "****"
      + "****" + "***" + "**");

  public static final BorderStyle HORIZONTAL_ONLY = new BorderStyle("", "-",
      " ", "", "", "-", " ", "", "", "-", " ", "", "", " ", "", " ", " ");

  // Classic tiles

  public static final BorderStyle CLASSIC = new BorderStyle("+-++" + "+-++"
      + "+-++" + "|||" + "--");

  public static final BorderStyle CLASSIC_WIDE = new BorderStyle("+-", "-",
      "-+-", "-+", "+-", "-", "-+-", "-+", "+-", "-", "-+-", "-+", "| ", " | ",
      " |", "---", "---");

  public static final BorderStyle CLASSIC_LIGHT = new BorderStyle("+--+"
      + "+--+" + "+--+" + "| |" + "--");

  public static final BorderStyle CLASSIC_LIGHT_WIDE = new BorderStyle("+-",
      "-", "--", "-+", "+-", "-", "--", "-+", "+-", "-", "--", "-+", "| ",
      "  ", " |", "--", "--");

  public static final BorderStyle CLASSIC_COMPATIBLE = new BorderStyle("+-++"
      + "+-++" + "+-++" + "!!!" + "--");

  public static final BorderStyle CLASSIC_COMPATIBLE_WIDE = new BorderStyle(
      "+-", "-", "-+-", "-+", "+-", "-", "-+-", "-+", "+-", "-", "-+-", "-+",
      "! ", " ! ", " !", "---", "---");

  public static final BorderStyle CLASSIC_COMPATIBLE_LIGHT_WIDE = new BorderStyle(
      "+-", "-", "--", "-+", "+-", "-", "--", "-+", "+-", "-", "--", "-+",
      "! ", "  ", " !", "--", "--");

  // Heavy tiles

  public static final BorderStyle HEAVY = new BorderStyle("+==+" + "+==+"
      + "+==+" + "|||" + "==");

  public static final BorderStyle HEAVY_TOP_AND_BOTTOM = new BorderStyle("+==+"
      + "+--+" + "+==+" + "|||" + "--");

  // Design tiles

  public static final BorderStyle DESIGN_FORMAL = new BorderStyle("", "=", "=",
      "", "", "-", " ", "", "", "=", "=", "", "", " ", "", " ", " ");

  public static final BorderStyle DESIGN_FORMAL_INVERSE = new BorderStyle("",
      "-", "-", "", "", "=", " ", "", "", "-", "-", "", "", " ", "", " ", " ");

  public static final BorderStyle DESIGN_CASUAL = new BorderStyle("", "=", "=",
      "", "", "~", " ", "", "", "=", "=", "", "", " ", "", " ", " ");

  public static final BorderStyle DESIGN_CAFE = new BorderStyle("", "~", "~",
      "", "", "~", " ", "", "", "~", "~", "", "", " ", "", " ", " ");

  public static final BorderStyle DESIGN_SLASH = new BorderStyle("", "/", "/",
      "", "", "-", " ", "", "", "/", "/", "", "", " ", "", " ", " ");

  public static final BorderStyle DESIGN_TUBES = new BorderStyle(" __ "
      + "|_||" + "|_||" + "|||" + "__");

  public static final BorderStyle DESIGN_DOTS = new BorderStyle("\u00b7",
      "\u00b7", "\u00b7", "\u00b7", "\u00b7", "\u00b7", "\u00b7", "\u00b7",
      "\u00b7", "\u00b7", "\u00b7", "\u00b7", "\u00b7", "\u00b7", "\u00b7",
      "\u00b7", "\u00b7");

  public static final BorderStyle DESIGN_DIM = new BorderStyle("", "\u00a8",
      "\u00a8", "", "", "\u00a8", " ", "", "", "\u00a8", "\u00a8", "", "", " ",
      "", "\u00a8", "\u00a8");

  public static final BorderStyle DESIGN_CURTAIN = new BorderStyle("o~", "~",
      "~", "~o", "  ", "-", " ", "  ", "o~", "~", "~", "~o", "  ", " ", "  ",
      " ", " ");

  public static final BorderStyle DESIGN_CURTAIN_HEAVY = new BorderStyle("o=",
      "=", "=", "=o", "  ", "-", " ", "  ", "o=", "=", "=", "=o", "  ", " ",
      "  ", " ", " ");

  public static final BorderStyle DESIGN_PAPYRUS = new BorderStyle("o~~~", "~",
      "~~", "~~~o", " )  ", "~", "  ", "  (", "o~~~", "~", "~~", "~~~o",
      " )  ", "  ", "  (", "  ", "  ");

  // Design wide

  public static final BorderStyle DESIGN_FORMAL_WIDE = new BorderStyle("", "=",
      "==", "", "", "-", "  ", "", "", "=", "==", "", "", "  ", "", "  ", "  ");

  public static final BorderStyle DESIGN_FORMAL_INVERSE_WIDE = new BorderStyle(
      "", "-", "--", "", "", "=", "  ", "", "", "-", "--", "", "", "  ", "",
      "  ", "  ");

  public static final BorderStyle DESIGN_CASUAL_WIDE = new BorderStyle("", "=",
      "==", "", "", "~", "  ", "", "", "=", "==", "", "", "  ", "", "  ", "  ");

  public static final BorderStyle DESIGN_CAFE_WIDE = new BorderStyle("", "~",
      "~~", "", "", "~", "  ", "", "", "~", "~~", "", "", "  ", "", "  ", "  ");

  public static final BorderStyle DESIGN_SLASH_WIDE = new BorderStyle("", "/",
      "//", "", "", "-", "  ", "", "", "/", "//", "", "", "  ", "", "  ", "  ");

  public static final BorderStyle DESIGN_TUBES_WIDE = new BorderStyle(" _",
      "_", "___", "_ ", "|_", "_", "_|_", "_|", "|_", "_", "_|_", "_|", "| ",
      " | ", " |", "___", "___");

  public static final BorderStyle DESIGN_DOTS_WIDE = new BorderStyle(
      "\u00b7\u00b7", "\u00b7", "\u00b7\u00b7\u00b7", "\u00b7\u00b7",
      "\u00b7\u00b7", "\u00b7", "\u00b7\u00b7\u00b7", " \u00b7",
      "\u00b7\u00b7", "\u00b7", "\u00b7\u00b7\u00b7", "\u00b7\u00b7",
      "\u00b7 ", " \u00b7 ", " \u00b7", "\u00b7\u00b7\u00b7",
      "\u00b7\u00b7\u00b7");

  public static final BorderStyle DESIGN_DIM_WIDE = new BorderStyle("",
      "\u00a8", "\u00a8\u00a8", "", "", "\u00a8", "  ", "", "", "\u00a8",
      "\u00a8\u00a8", "", "", "  ", "", "  ", "  ");

  public static final BorderStyle DESIGN_CURTAIN_WIDE = new BorderStyle("o~",
      "~", "~~", "~o", "  ", "-", "  ", "  ", "o~", "~", "~~", "~o", "  ",
      "  ", "  ", "  ", "  ");

  public static final BorderStyle DESIGN_CURTAIN_HEAVY_WIDE = new BorderStyle(
      "o=", "=", "==", "=o", "  ", "-", "  ", "  ", "o=", "=", "==", "=o",
      "  ", "  ", "  ", "  ", "  ");

  // Unicode tiles

  public static final BorderStyle UNICODE_BOX = new BorderStyle(
      "\u250c\u2500\u252c\u2510" + "\u251c\u2500\u253c\u2524"
          + "\u2514\u2500\u2534\u2518" + "\u2502\u2502\u2502" + "\u2534\u252c");

  public static final BorderStyle UNICODE_ROUND_BOX = new BorderStyle(
      "\u256d\u2500\u252c\u256e" + "\u251c\u2500\u253c\u2524"
          + "\u2570\u2500\u2534\u256f" + "\u2502\u2502\u2502" + "\u2534\u252c");

  public static final BorderStyle UNICODE_HEAVY_BOX = new BorderStyle(
      "\u250f\u2501\u2533\u2513" + "\u2523\u2501\u254b\u252b"
          + "\u2517\u2501\u253b\u251b" + "\u2503\u2503\u2503" + "\u253b\u2533");

  public static final BorderStyle UNICODE_BOX_HEAVY_BORDER = new BorderStyle(
      "\u250f\u2501\u252f\u2513" + "\u2520\u2500\u253c\u2528"
          + "\u2517\u2501\u2537\u251b" + "\u2503\u2502\u2503" + "\u2534\u252c");

  public static final BorderStyle UNICODE_DOUBLE_BOX = new BorderStyle(
      "\u2554\u2550\u2566\u2557" + "\u2560\u2550\u256c\u2563"
          + "\u255a\u2550\u2569\u255d" + "\u2551\u2551\u2551" + "\u2569\u2566");

  public static final BorderStyle UNICODE_BOX_DOUBLE_BORDER = new BorderStyle(
      "\u2554\u2550\u2564\u2557" + "\u255f\u2500\u253c\u2562"
          + "\u255a\u2550\u2567\u255d" + "\u2551\u2502\u2551" + "\u2534\u252c");

  // Unicode wide tiles

  public static final BorderStyle UNICODE_BOX_WIDE = new BorderStyle(
      "\u250c\u2500", "\u2500", "\u2500\u252c\u2500", "\u2500\u2510",
      "\u251c\u2500", "\u2500", "\u2500\u253c\u2500", "\u2500\u2524",
      "\u2514\u2500", "\u2500", "\u2500\u2534\u2500", "\u2500\u2518",
      "\u2502 ", " \u2502 ", " \u2502", "\u2500\u2534\u2500",
      "\u2500\u252c\u2500");

  public static final BorderStyle UNICODE_ROUND_BOX_WIDE = new BorderStyle(
      "\u256d\u2500", "\u2500", "\u2500\u252c\u2500", "\u2500\u256e",
      "\u251c\u2500", "\u2500", "\u2500\u253c\u2500", "\u2500\u2524",
      "\u2570\u2500", "\u2500", "\u2500\u2534\u2500", "\u2500\u256f",
      "\u2502 ", " \u2502 ", " \u2502", "\u2500\u2534\u2500",
      "\u2500\u252c\u2500");

  public static final BorderStyle UNICODE_HEAVY_BOX_WIDE = new BorderStyle(
      "\u250f\u2501", "\u2501", "\u2501\u2533\u2501", "\u2501\u2513",
      "\u2523\u2501", "\u2501", "\u2501\u254b\u2501", "\u2501\u252b",
      "\u2517\u2501", "\u2501", "\u2501\u253b\u2501", "\u2501\u251b",
      "\u2503 ", " \u2503 ", " \u2503", "\u2501\u253b\u2501",
      "\u2501\u2533\u2501");

  public static final BorderStyle UNICODE_BOX_HEAVY_BORDER_WIDE = new BorderStyle(
      "\u250f\u2501", "\u2501", "\u2501\u252f\u2501", "\u2501\u2513",
      "\u2520\u2500", "\u2500", "\u2500\u253c\u2500", "\u2500\u2528",
      "\u2517\u2501", "\u2501", "\u2501\u2537\u2501", "\u2501\u251b",
      "\u2503 ", " \u2502 ", " \u2503", "\u2500\u2534\u2500",
      "\u2500\u252c\u2500");

  public static final BorderStyle UNICODE_DOUBLE_BOX_WIDE = new BorderStyle(
      "\u2554\u2550", "\u2550", "\u2550\u2566\u2550", "\u2550\u2557",
      "\u2560\u2550", "\u2550", "\u2550\u256c\u2550", "\u2550\u2563",
      "\u255a\u2550", "\u2550", "\u2550\u2569\u2550", "\u2550\u255d",
      "\u2551 ", " \u2551 ", " \u2551", "\u2550\u2569\u2550",
      "\u2550\u2566\u2550");

  public static final BorderStyle UNICODE_BOX_DOUBLE_BORDER_WIDE = new BorderStyle(
      "\u2554\u2550", "\u2550", "\u2550\u2564\u2550", "\u2550\u2557",
      "\u255f\u2500", "\u2500", "\u2500\u253c\u2500", "\u2500\u2562",
      "\u255a\u2550", "\u2550", "\u2550\u2567\u2550", "\u2550\u255d",
      "\u2551 ", " \u2502 ", " \u2551", "\u2500\u2534\u2500",
      "\u2500\u252c\u2500");

  // Top

  private String tLCorner;

  private String top;

  private String tCCorner;

  private String tRCorner;

  // Middle

  private String mLCorner;

  private String middle;

  private String mCCorner;

  private String mRCorner;

  // Bottom

  private String bLCorner;

  private String bottom;

  private String bCCorner;

  private String bRCorner;

  // Vertical separators

  private String left;

  private String center;

  private String right;

  // column span characters

  private String upperColSpan;

  private String lowerColSpan;

  // tile widths

  private int leftWidth;

  private int horizontalWidth;

  private int centerWidth;

  private int rightWidth;

  /**
   * Specifies a border style with multi-character tiles.
   * 
   * @param tLCorner
   *          Top left corner
   * @param top
   *          Top
   * @param tCCorner
   *          Top center corner
   * @param tRCorner
   *          Top right corner
   * @param mLCorner
   *          Middle left corner
   * @param middle
   *          Middle
   * @param mCCorner
   *          Middle center corner
   * @param mRCorner
   *          Middle right corner
   * @param bLCorner
   *          Bottom left corner
   * @param bottom
   *          Bottom
   * @param bCCorner
   *          Bottom center corner
   * @param bRCorner
   *          Bottom right corner
   * @param left
   *          Left
   * @param center
   *          Center
   * @param right
   *          Right
   * @param upperColSpan
   *          middle center corner for the upper border of a colspan
   * @param lowerColSpan
   *          middle center corner for the lower border of a colspan
   */
  public BorderStyle(final String tLCorner, final String top,
      final String tCCorner, final String tRCorner, final String mLCorner,
      final String middle, final String mCCorner, final String mRCorner,
      final String bLCorner, final String bottom, final String bCCorner,
      final String bRCorner, final String left, final String center,
      final String right, final String upperColSpan, final String lowerColSpan) {

    this.leftWidth = maxWidth(tLCorner, mLCorner, bLCorner, left);
    this.horizontalWidth = 1; // maxWidth(top, middle, bottom);
    this.centerWidth = maxWidth(tCCorner, mCCorner, bCCorner, center,
        upperColSpan, lowerColSpan);
    this.rightWidth = maxWidth(tRCorner, mRCorner, bRCorner, right);

    this.tLCorner = adjustString(tLCorner, this.leftWidth);
    this.top = adjustString(top, this.horizontalWidth);
    this.tCCorner = adjustString(tCCorner, this.centerWidth);
    this.tRCorner = adjustString(tRCorner, this.rightWidth);
    this.mLCorner = adjustString(mLCorner, this.leftWidth);
    this.middle = adjustString(middle, this.horizontalWidth);
    this.mCCorner = adjustString(mCCorner, this.centerWidth);
    this.mRCorner = adjustString(mRCorner, this.rightWidth);
    this.bLCorner = adjustString(bLCorner, this.leftWidth);
    this.bottom = adjustString(bottom, this.horizontalWidth);
    this.bCCorner = adjustString(bCCorner, this.centerWidth);
    this.bRCorner = adjustString(bRCorner, this.rightWidth);
    this.left = adjustString(left, this.leftWidth);
    this.center = adjustString(center, this.centerWidth);
    this.right = adjustString(right, this.rightWidth);
    this.upperColSpan = adjustString(upperColSpan, this.centerWidth);
    this.lowerColSpan = adjustString(lowerColSpan, this.centerWidth);
  }

  private int maxWidth(final String a, final String b) {
    return Math.max(tileWidth(a), tileWidth(b));
  }

  private int maxWidth(final String a, final String b, final String c) {
    return Math.max(Math.max(tileWidth(a), tileWidth(b)), tileWidth(c));
  }

  private int maxWidth(final String a, final String b, final String c,
      final String d) {
    return Math.max(Math
        .max(Math.max(tileWidth(a), tileWidth(b)), tileWidth(c)), tileWidth(d));
  }

  private int maxWidth(final String a, final String b, final String c,
      final String d, final String e, final String f) {
    return Math.max(maxWidth(a, b, c), maxWidth(d, e, f));
  }

  private String adjustString(final String txt, final int width) {
    if (txt == null) {
      return Filler.getFiller(width);
    }
    if (txt.length() == width) {
      return txt;
    }
    if (txt.length() > width) {
      return txt.substring(0, width);
    }
    int diff = width - txt.length();
    return txt + Filler.getFiller(diff);
  }

  private int tileWidth(final String tile) {
    if (tile == null) {
      return 0;
    }
    return tile.length();
  }

  /**
   * Specifies a border style with single-character tiles.
   * 
   * @param customStyle
   *          15-character String where every character is a tile, in the
   *          following order: <ul> <li>Top left corner</li> <li>Top</li>
   *          <li>Top center corner</li> <li>Top right corner</li> <li>Middle
   *          left corner</li> <li>Middle</li> <li>Middle center corner</li>
   *          <li>Middle right corner</li> <li>Bottom left corner</li> <li>Bottom</li>
   *          <li>Bottom center corner</li> <li>Bottom right corner</li>
   *          <li>Left</li> <li>Center</li> <li>Right</li> 
   *          <li>Upper colspan</li> 
   *          <li>Lower colspan</li> 
   *          </ul>
   */
  public BorderStyle(final String customStyle) {
    this.leftWidth = 1;
    this.horizontalWidth = 1;
    this.centerWidth = 1;
    this.rightWidth = 1;

    this.tLCorner = get(customStyle, 0);
    this.top = get(customStyle, 1);
    this.tCCorner = get(customStyle, 2);
    this.tRCorner = get(customStyle, 3);
    this.mLCorner = get(customStyle, 4);
    this.middle = get(customStyle, 5);
    this.mCCorner = get(customStyle, 6);
    this.mRCorner = get(customStyle, 7);
    this.bLCorner = get(customStyle, 8);
    this.bottom = get(customStyle, 9);
    this.bCCorner = get(customStyle, 10);
    this.bRCorner = get(customStyle, 11);
    this.left = get(customStyle, 12);
    this.center = get(customStyle, 13);
    this.right = get(customStyle, 14);
    this.upperColSpan = get(customStyle, 15);
    this.lowerColSpan = get(customStyle, 16);
  }

  private static final String DEFAULT_TILE = "*";

  private String get(final String style, final int index) {
    if (style == null) {
      return DEFAULT_TILE;
    }
    if (index < 0 || index >= style.length()) {
      return DEFAULT_TILE;
    }
    return style.substring(index, index + 1);
  }

  // Accessors

  public String getBCCorner() {
    return this.bCCorner;
  }

  public String getBLCorner() {
    return this.bLCorner;
  }

  public String getBottom() {
    return this.bottom;
  }

  public String getBRCorner() {
    return this.bRCorner;
  }

  public String getCenter() {
    return this.center;
  }

  public String getLeft() {
    return this.left;
  }

  public String getMCCorner() {
    return this.mCCorner;
  }

  public String getMiddle() {
    return this.middle;
  }

  public String getMLCorner() {
    return this.mLCorner;
  }

  public String getMRCorner() {
    return this.mRCorner;
  }

  public String getRight() {
    return this.right;
  }

  public String getTop() {
    return this.top;
  }

  public String getTCCorner() {
    return this.tCCorner;
  }

  public String getTLCorner() {
    return this.tLCorner;
  }

  public String getTRCorner() {
    return this.tRCorner;
  }

  public String getUpperColSpan() {
    return this.upperColSpan;
  }

  public String getLowerColSpan() {
    return this.lowerColSpan;
  }

  public int getLeftWidth() {
    return leftWidth;
  }

  public int getHorizontalWidth() {
    return horizontalWidth;
  }

  public int getCenterWidth() {
    return centerWidth;
  }

  public int getRightWidth() {
    return rightWidth;
  }

}
